/***************************************************************************
 *   Copyright (C) 2003 by Matteo Merli                                    *
 *   matteo@petaligrigi.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
#include "actions.h"
#include "my_header.h"
#include "my_label.h"
#include "view.h"

#include <kfileitem.h>
#include <kapplication.h>
#include <kdebug.h>
#include <klocale.h>
#include <dcopclient.h>
#include <dcopref.h>
#include <kglobalsettings.h>
#include <kglobal.h>
#include <kstandarddirs.h> 
#include <qdir.h>

#include <konq_operations.h>

#define FILE 1
#define DIR  2

#define LINK 1
#define KDE_ACTION 2
#define DCOP_ACTION 3
#define SLOT_ACTION 4
#define KDE_SERVICE 5

/*
struct s_action 
{
	QString text;
	QString icon;
	int act_type;
	QString action;
	
	QString mime_type;
	int type;
};
*/
s_action action[] = 
{ 
	{ I18N_NOOP("Send file by mail"), "mail_send", KDE_ACTION, "sendPage", "", FILE, NULL},
//	{ i18n("Properties"), "info", KDE_ACTION, "properties", "", FILE, NULL},
	{ QString::null, "", 0, "", "", 0, NULL}
};

s_action action_trash = {
	I18N_NOOP("Empty Trash"), "trashcan_empty", SLOT_ACTION, "emptyTrash", "", 0, NULL};
s_action action_newdir = {
	I18N_NOOP("Create new directory"), "folder", SLOT_ACTION, "newDir", "", DIR, NULL};

Actions::Actions(QWidget *parent, QBoxLayout *layout,
		 QString text, QString icon)
{
	mParent = (View *)parent;
	header = new MyHeader(mParent, layout, text, icon);

	/* This one shows the current mime-type and is here
	   only for test.. */
	// label = new MyLabel(mParent, header, "", 
	//		    "", "");
	// label->hide();
	
	/*
	for (int i=0; i<6; i++ ) {
		act_label[i] = new MyLabel(mParent, header, "", "", "");
		act_label[i]->hide();	
		connect( act_label[i], SIGNAL(click_action(s_action &)), 
			 this, SLOT(clicked(s_action &)));
	}
	*/
		
	connect( mParent, SIGNAL( urlChanged(const KURL &) ),
		 this, SLOT( urlChanged(const KURL &) ) );
	
	// mLabelList.setAutoDelete( true );
	
	QByteArray data, rec_data;
	QDataStream stream( rec_data, IO_ReadOnly );
	QCString type;
	QCString id = kapp->dcopClient()->appId();
	
	if( id.isEmpty() ) {
		kdDebug() << "Error in DCOP appId().\n";
		return;
	}
	
	if( !kapp->dcopClient()->call( 
	        id, "konqueror-mainwindow#1", "currentURL()", data,
		type, rec_data ) ) {
		kdDebug() << "Error in DCOP currentURL() send.\n";
		return;
	}
	
	QString str;
	stream >> str;
	this->urlChanged( KURL( str ) );
}


Actions::~Actions()
{
}

void Actions::urlChanged(const KURL &url)
{
	int type;
	QString mime_type;
	
	mUrl = url; 
	
	KFileItem *file = new KFileItem( KFileItem::Unknown, KFileItem::Unknown, 
					 mUrl, false);
	
	if ( file->isFile() )
		type = FILE;
	else if ( file->isDir() )
		type = DIR;
	else type = 0;
	
	mime_type = file->mimetype();
	
	kdDebug() << "mLabelList.count() : " << mLabelList.count() << "\n";
	
	for ( int i=0; mLabelList.count() > 0; ) {
		kdDebug() << "Deleting \n";
		MyLabel *item = mLabelList.take( i );
		item->hide();
		delete item;
	}
		
	kdDebug() << "mLabelList.count() : " << mLabelList.count() << "\n";

	if ( ! (url.protocol() != "file") ) {
		if ( mUrl.path() == KGlobalSettings::trashPath() ) {
			if ( QDir(mUrl.path()).count() > 2 ) {
				MyLabel *label = new MyLabel( mParent, header, 
						action_trash.text, "", action_trash.icon);
				label->setAction( action_trash );
				connect( label, SIGNAL(click_action(s_action &)), 
			 		this, SLOT(clicked(s_action &)));
				label->show();
				mLabelList.append( label );
			}
		} else if ( file->isDir() ){
			MyLabel *label = new MyLabel( mParent, header, 
					action_newdir.text, "", action_newdir.icon);
			label->setAction( action_newdir );
			connect( label, SIGNAL(click_action(s_action &)), 
				this, SLOT(clicked(s_action &)));
			label->show();
			mLabelList.append( label );
		}
		
		for (int i=0; action[i].text != QString::null; i++) {
			if ( action[i].type == type || 
			!QString::compare(action[i].mime_type, mime_type) ) {
				
				MyLabel *label = new MyLabel( mParent, header, 
						action[i].text, "", action[i].icon);
				label->setAction( action[i] );
				connect( label, SIGNAL(click_action(s_action &)), 
			 		this, SLOT(clicked(s_action &)));
				label->show();
				mLabelList.append( label );
			}
		}
	}
	
	loadServices( file );
	kdDebug() << "mDeviceServices.count() : " << mDeviceServices.count() << "\n";
	kdDebug() << "mDirectoryServices.count() : " << mDirectoryServices.count() << "\n";

	if ( !mDeviceServices.isEmpty() ) {
		QValueList<KDEDesktopMimeType::Service>::Iterator it= mDeviceServices.begin();
		for( ; it != mDeviceServices.end(); ++it ) {
			MyLabel *label = new MyLabel(mParent, header, (*it).m_strName, "", (*it).m_strIcon);
			label->show();
			s_action action = { "", "", KDE_SERVICE, file->url().path(), "", 0, (void *)&(*it)};
			label->setAction( action );
			connect( label, SIGNAL(click_action(s_action &)), 
				 this, SLOT( kdeService(s_action &)));
			mLabelList.append( label );
		}
	}
	
	if ( !mDirectoryServices.isEmpty() ) {
		QValueList<KDEDesktopMimeType::Service>::Iterator it= mDirectoryServices.begin();
		for( ; it != mDirectoryServices.end(); ++it ) {
			MyLabel *label = new MyLabel(mParent, header, (*it).m_strName, "", (*it).m_strIcon);
			label->show();
			// connect( label, SIGNAL(click_action(s_action &)), 
			//	 this, SLOT(clicked(s_action &)));
			mLabelList.append( label );
		}
	}

	
	if ( mLabelList.count() == 0 )
		/* We should hide the whole Actions panel */
		header->hide();
	else	header->show();
	
	delete file;
}

void Actions::clicked(s_action &act)
{
	switch (act.act_type) {
		case LINK:        link( act.action );
				  return;
		case KDE_ACTION:  kdeAction( act.action );
				  return;
		case DCOP_ACTION: dcopAction( act.action );
			  	return;
		case SLOT_ACTION: 
			if ( act.action == "emptyTrash")
				KonqOperations::emptyTrash();
			else if (act.action == "newDir")
#if KDE_VERSION_MINOR == 1 && KDE_VERSION_RELEASE < 92
				KonqOperations::mkDir( (QWidget *)0, mUrl);
#else
				KonqOperations::newDir( (QWidget *)0, mUrl);
#endif
		default:	break;
	};
	
	
}

void Actions::kdeService(s_action &act )
{
	KDEDesktopMimeType::Service service = *(KDEDesktopMimeType::Service *)(act.service);
	KURL url ;// = mMountPoint;

	// if ( mDeviceServices.contains( service ) ) {
		url = KURL( act.action );// mFileItem.url();
	// }

	KDEDesktopMimeType::executeService( url, service );
}

void Actions::dcopAction(QString &act)
{
	kdDebug() << " ******** void Actions::dcopAction( " << act << " ) *******\n";
}


/* We call a konqueror action using the
   dcop interface. */
void Actions::kdeAction(QString &act)
{
	
	kdDebug() << " ******** void Actions::kdeAction( " << act << " ) *******\n";
	QByteArray data, rec_data;
	QDataStream stream1( data, IO_WriteOnly );
	QDataStream stream2( rec_data, IO_ReadOnly );
	stream1 << (QCString)act;
	QCString type;
	QCString id = kapp->dcopClient()->appId();
	
	if( id.isEmpty() ) {
		kdDebug() << "Error in DCOP appId().\n";
		return;
	}
	
	if( !kapp->dcopClient()->call( 
	        id, "konqueror-mainwindow#1", "activateAction(QCString)", 
		data, type, rec_data ) ) {
		kdDebug() << "Error in DCOP currentURL() send.\n";
		return;
	}
}

void Actions::link(QString &act)
{
	kdDebug() << " ******** void Actions::link( " << act << " ) *******\n";
	mParent->clicked( act );
}


void Actions::loadServices(KFileItem *file)
{
	QStringList dirs = KGlobal::dirs()->findDirs("data", "konqueror/servicemenus/");
	QStringList::ConstIterator dIt = dirs.begin();
	QStringList::ConstIterator dEnd = dirs.end();
	
	mDeviceServices.clear();
	mDirectoryServices.clear();

	for ( ; dIt != dEnd; ++dIt)
	{
		QDir dir(*dIt);

		QStringList entries = dir.entryList(QDir::Files);
		QStringList::ConstIterator eIt = entries.begin();
		QStringList::ConstIterator eEnd = entries.end();

		for ( ; eIt != eEnd; ++eIt) {
			KSimpleConfig cfg(*dIt+*eIt, true);
			cfg.setDesktopGroup();

			if ( cfg.hasKey("X-KDE-AuthorizeAction") ) {
				bool ok = true;
				QStringList list = cfg.readListEntry("X-KDE-AuthorizeAction");

				if ( kapp && !list.isEmpty() ) {
					for(QStringList::ConstIterator it = list.begin();
					    it != list.end();
					    ++it) {
						if (!kapp->authorize((*it).stripWhiteSpace())) {
							ok = false;
							break;
						}
					}
				}

				if (!ok) 
					continue;
			}

			if (cfg.hasKey( "Actions" ) && cfg.hasKey( "ServiceTypes" )) {
				QStringList types = cfg.readListEntry("ServiceTypes");
				if( types.contains(file->mimetype()) ) {
					mDeviceServices += KDEDesktopMimeType::userDefinedServices(*dIt+*eIt, true);
				}
				else if( types.contains("inode/directory") ) {
					mDirectoryServices += 
						KDEDesktopMimeType::userDefinedServices(*dIt+*eIt, true);
				}
			}
		}
	}
}




#include "actions.moc"
