/***************************************************************************
 *   Copyright (C) 2003 by Matteo Merli                                    *
 *   matteo@petaligrigi.net                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "view.h"
#include "my_label.h"
#include "my_header.h"
#include "details.h"
#include "actions.h"
#include "devices.h"

#include <qlayout.h>
#include <qlabel.h>
#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kurllabel.h>
#include <kapplication.h>
#include <kdebug.h>
#include <dcopclient.h>
#include <dcopref.h>
#include <qscrollview.h>
#include <kconfig.h>

#include <kglobalsettings.h>

//extern const char* const image_data[];


View::View(QWidget *parent) 
	: QFrame( parent , "View")
	 // image( ( (const char **) image_data ) )
	 
{
	setFrameStyle( QFrame::Panel );
	// setPaletteBackgroundPixmap( image );
	
	config = new KConfig("ksidebarrc", false, false);
	
	QVBoxLayout *mainLayout = new QVBoxLayout( this, 7, 0 );
	
	//////////////////////////////////////////////////////////////
	head_actions = new Actions(this, mainLayout, i18n("Actions"), "exec");
	head_actions->header->setVisible( config->readBoolEntry("Actions", true ));
	
	///////////////////////////////////////////////////////////////
	head_devices = new Devices(this, mainLayout, 
				i18n("Devices"), "kcmdevices");
	/*
	head_devices = new MyHeader(this, mainLayout, 
					      i18n("Devices"), "misc");
	head_devices->setVisible( config->readBoolEntry("Devices", true ));
	new MyLabel(this, head_devices, i18n("CD"), 
		    "file:/mnt/cdrom", "cdrom_unmount");
	new MyLabel(this, head_devices, i18n("Floppy"),
		    "floppy:/", "3floppy_unmount");
	new MyLabel(this, head_devices, i18n("Printers"),
		    "print:/", "print_printer");
	new MyLabel(this, head_devices, i18n("All devices"),
		    "devices:/", "block_device");
	*/
	///////////////////////////////////////////////////////////////
	head_shortcuts = new MyHeader(this, mainLayout,
						i18n("Shortcuts"), "launch");
	head_shortcuts->setVisible( config->readBoolEntry("Shortcuts", true ));
	new MyLabel(this, head_shortcuts, i18n("Home"), 
		    "~", "gohome");
	new MyLabel(this, head_shortcuts, i18n("Documents"), 
		    KGlobalSettings::documentPath(), "folder_txt");
	new MyLabel(this, head_shortcuts, i18n("Trash"), 
		    KGlobalSettings::trashPath(), "trashcan_full");
	new MyLabel(this, head_shortcuts, i18n("Settings"), 
		    "settings:/", "kcontrol");
	
	/////////////////////////////////////////////////////////////////////
	head_details = new Details( this, mainLayout, i18n("Details"), "info");
	head_details->header->setVisible( config->readBoolEntry("Details", true ));
	
	mainLayout->addStretch(); 
}

View::~View()
{
	// Here we should save all the config data 
	config->writeEntry("Actions", head_actions->header->isVisible() );
	config->writeEntry("Devices", head_devices->header->isVisible() );
	config->writeEntry("Shortcuts", head_shortcuts->isVisible() );
	config->writeEntry("Details", head_details->header->isVisible() );
	config->sync();
}

/*
   This code for the DCOP call was taken from
   KontextBar by domine <domine@paranoici.org> 
   http://www.kde-look.org/content/show.php?content=8015
 */
void View::clicked( const QString &url )
{
	QByteArray data;
	QDataStream stream( data, IO_WriteOnly );
	stream << url;
	
	QCString id = kapp->dcopClient()->appId();
	
	if( id.isEmpty() ) {
		kdDebug() << "Error in DCOP appId().\n";
		return;
	}
	
	if( !kapp->dcopClient()->send( id, "konqueror-mainwindow#1",
		"openURL(QString)", data ) ) {
		kdDebug() << "Error in DCOP openURL() send.\n";
		return;
	}
}

#include "view.moc"
